"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessagePickupApi = void 0;
const agent_1 = require("../../agent");
const MessageSender_1 = require("../../agent/MessageSender");
const models_1 = require("../../agent/models");
const constants_1 = require("../../constants");
const error_1 = require("../../error");
const plugins_1 = require("../../plugins");
const services_1 = require("../connections/services");
const MessagePickupModuleConfig_1 = require("./MessagePickupModuleConfig");
const MessagePickupSessionService_1 = require("./services/MessagePickupSessionService");
let MessagePickupApi = class MessagePickupApi {
    constructor(messageSender, agentContext, connectionService, messagePickupSessionService, config, logger) {
        this.messageSender = messageSender;
        this.connectionService = connectionService;
        this.agentContext = agentContext;
        this.config = config;
        this.messagePickupSessionService = messagePickupSessionService;
        this.logger = logger;
    }
    async initialize() {
        this.messagePickupSessionService.start(this.agentContext);
    }
    getProtocol(protocolVersion) {
        const protocol = this.config.protocols.find((protocol) => protocol.version === protocolVersion);
        if (!protocol) {
            throw new error_1.CredoError(`No message pickup protocol registered for protocol version ${protocolVersion}`);
        }
        return protocol;
    }
    /**
     * Add an encrypted message to the message pickup queue
     *
     * @param options: connectionId associated to the message and the encrypted message itself
     */
    async queueMessage(options) {
        this.logger.debug('Queuing message...');
        const { connectionId, message, recipientDids } = options;
        const connectionRecord = await this.connectionService.getById(this.agentContext, connectionId);
        const messagePickupRepository = this.agentContext.dependencyManager.resolve(constants_1.InjectionSymbols.MessagePickupRepository);
        await messagePickupRepository.addMessage({ connectionId: connectionRecord.id, recipientDids, payload: message });
    }
    /**
     * Get current active live mode message pickup session for a given connection. Undefined if no active session found
     *
     * @param options connection id and optional role
     * @returns live mode session
     */
    async getLiveModeSession(options) {
        const { connectionId, role } = options;
        return this.messagePickupSessionService.getLiveSessionByConnectionId(this.agentContext, { connectionId, role });
    }
    /**
     * Deliver specific messages to an active live mode pickup session through message pickup protocol.
     *
     * This will deliver the messages regardless of the state of the message pickup queue, meaning that
     * any message stuck there should be sent separately (e.g. using deliverQU).
     *
     * @param options: pickup session id and the messages to deliver
     */
    async deliverMessages(options) {
        const { pickupSessionId, messages } = options;
        const session = this.messagePickupSessionService.getLiveSession(this.agentContext, pickupSessionId);
        if (!session) {
            this.logger.debug(`No active live mode session found with id ${pickupSessionId}`);
            return;
        }
        const connectionRecord = await this.connectionService.getById(this.agentContext, session.connectionId);
        const protocol = this.getProtocol(session.protocolVersion);
        const createDeliveryReturn = await protocol.createDeliveryMessage(this.agentContext, {
            connectionRecord,
            messages,
        });
        if (createDeliveryReturn) {
            await this.messageSender.sendMessage(new models_1.OutboundMessageContext(createDeliveryReturn.message, {
                agentContext: this.agentContext,
                connection: connectionRecord,
            }));
        }
    }
    /**
     * Deliver messages in the Message Pickup Queue for a given live mode session and key (if specified).
     *
     * This will retrieve messages up to 'batchSize' messages from the queue and deliver it through the
     * corresponding Message Pickup protocol. If there are more than 'batchSize' messages in the queue,
     * the recipient may request remaining messages after receiving the first batch of messages.
     *
     */
    async deliverMessagesFromQueue(options) {
        this.logger.debug('Deliverying queued messages');
        const { pickupSessionId, recipientDid: recipientKey, batchSize } = options;
        const session = this.messagePickupSessionService.getLiveSession(this.agentContext, pickupSessionId);
        if (!session) {
            throw new error_1.CredoError(`No active live mode session found with id ${pickupSessionId}`);
        }
        const connectionRecord = await this.connectionService.getById(this.agentContext, session.connectionId);
        const protocol = this.getProtocol(session.protocolVersion);
        const deliverMessagesReturn = await protocol.createDeliveryMessage(this.agentContext, {
            connectionRecord,
            recipientKey,
            batchSize,
        });
        if (deliverMessagesReturn) {
            await this.messageSender.sendMessage(new models_1.OutboundMessageContext(deliverMessagesReturn.message, {
                agentContext: this.agentContext,
                connection: connectionRecord,
            }));
        }
    }
    /**
     * Pickup queued messages from a message holder. It attempts to retrieve all current messages from the
     * queue, receiving up to `batchSize` messages per batch retrieval.
     *
     * @param options connectionId, protocol version to use and batch size
     */
    async pickupMessages(options) {
        const connectionRecord = await this.connectionService.getById(this.agentContext, options.connectionId);
        const protocol = this.getProtocol(options.protocolVersion);
        const { message } = await protocol.createPickupMessage(this.agentContext, {
            connectionRecord,
            batchSize: options.batchSize,
            recipientDid: options.recipientDid,
        });
        await this.messageSender.sendMessage(new models_1.OutboundMessageContext(message, {
            agentContext: this.agentContext,
            connection: connectionRecord,
        }));
    }
    /**
     * Enable or disable Live Delivery mode as a recipient. If there were previous queued messages, it will pick-up them
     * automatically.
     *
     * @param options connectionId, protocol version to use and boolean to enable/disable Live Mode
     */
    async setLiveDeliveryMode(options) {
        const connectionRecord = await this.connectionService.getById(this.agentContext, options.connectionId);
        const protocol = this.getProtocol(options.protocolVersion);
        const { message } = await protocol.setLiveDeliveryMode(this.agentContext, {
            connectionRecord,
            liveDelivery: options.liveDelivery,
        });
        await this.messageSender.sendMessage(new models_1.OutboundMessageContext(message, {
            agentContext: this.agentContext,
            connection: connectionRecord,
        }));
    }
};
MessagePickupApi = __decorate([
    (0, plugins_1.injectable)(),
    __param(5, (0, plugins_1.inject)(constants_1.InjectionSymbols.Logger)),
    __metadata("design:paramtypes", [MessageSender_1.MessageSender,
        agent_1.AgentContext,
        services_1.ConnectionService,
        MessagePickupSessionService_1.MessagePickupSessionService,
        MessagePickupModuleConfig_1.MessagePickupModuleConfig, Object])
], MessagePickupApi);
exports.MessagePickupApi = MessagePickupApi;
//# sourceMappingURL=MessagePickupApi.js.map